use strict;
use warnings;

package Apache::IPmanage;

use Apache::Constants qw(:common);
use Apache::Cookie;
use Apache::Request;
use IPmanage;

sub handler {
    my ($r, $uri, $path, $ip, $req, $cookie, $sesion, %params, $ret, $base);
    $r = shift;

    # Tomo de $r->uri y $r->path_info y masajeo para que en $uri me quede el
    # URI base del sistema y en $path el path dentro del mismo
    $uri = $r->uri();
    $path = $r->path_info();
    $path =~ s!^/!!;
    $uri =~ s!$path$!!;
    # Le agrego el / final si no lo tiene
    $uri .= '/' unless (substr($uri, length($uri)-1, 1) eq '/');

    $req = Apache::Request->new($r);
    $req->parse;
    $cookie = Apache::Cookie->fetch;
    $sesion = defined $cookie->{ipmanage} ? $cookie->{ipmanage}->value : '';

    for my $key (keys %{$req->param}) {
	my @values = $req->param($key);
	$params{$key} = scalar(@values) > 1 ? [@values] : $values[0];
    }

    $ip = IPmanage->new(accion => [split('/', $path)],
			uri_base => $uri,
			dbname => $r->dir_config('dbname'),
			dbuser => $r->dir_config('dbuser'),
			dbpass => $r->dir_config('dbpass'),
			dbhost => $r->dir_config('dbhost'),
			debug => $r->dir_config('debug'),
			tmpldir => $r->dir_config('tmpldir'),
			params => \%params,
			sesion => $sesion);

    $ip->process;

    if (my $sesion = $ip->sesion) {
	$cookie = Apache::Cookie->new($r, -name=>'ipmanage',
				      -value=>$sesion, -expires=>'+1h');
	$cookie->bake;
    }

    $r->send_http_header($ip->content_type);
    $r->print($ip->get_output);

    return OK;
}

1;
