use strict;
use warnings;

package Apache2::IPmanage;

=head1 NAME

Apache2::IPmanage - Mod_perl2 module to invoke IPmanage 

=head1 SYNOPSIS

To invoke this module from Apache, include the following directives (of course,
suited to your site) in your Apache configuration file or directory. This will
usually be located inside one of your virtualhost declarations:

    <Location /ipmanage>
        SetHandler perl-script
        PerlHandler Apache2::IPmanage

        PerlSetVar dbname  "ipmanage"
        PerlSetVar dbuser  "ipmanage"
        PerlSetVar dbpass  "dbpasswd"
        PerlSetVar dbhost  "db.example.com"
        PerlSetVar debug   "0"
        PerlSetVar lang    "en"
        PerlSetVar tmpldir "/home/gwolf/cvs/ipmanage/tmpl"
    </Location>

(remember to finish this... :-/)

=head1 REQUIRES

- Apache 2.0
- Perl 5.8
- Mod_perl 2.0
- (...remember to finish this...)

=head1 SEE ALSO

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms
as Perl.

=cut

use IPmanage;
use Apache2::Connection ();
use Apache2::Const qw(:common);
use Apache2::Request;
use Apache2::RequestRec;
use Apache2::RequestUtil;
use APR::URI;
use CGI::Cookie;
use Data::Dumper;

sub handler {
    my ($r, $uri, $path, $ip, $req, $cookie, $session, %params, $ret, $base, 
	$conn, $par_obj);
    $r = shift;
    $conn = $r->connection;

    # Get the URI from Apache got a real mess... Don't you hate mod_perl2? ;-)
    $uri = APR::URI->parse($r->pool, $r->uri)->path;
    $path = $r->path_info();
    $path =~ s!^/!!;
    $uri =~ s!$path$!!;
    $uri .= '/' unless (substr($uri, length($uri)-1, 1) eq '/');

    # Get the cookies and the parameters
    $req = Apache2::Request->new($r);
    $cookie = CGI::Cookie->fetch($r);
    $session = defined $cookie->{ipmanage} ? $cookie->{ipmanage}->value : '';

    $par_obj = $req->param || {};
    for my $key (keys %$par_obj) {
	my @values = $req->param($key);
	$params{$key} = scalar(@values) > 1 ? [@values] : $values[0];
    }

    # Create a IPmanage object and process the request
    $ip = IPmanage->new(action => [split('/', $path)],
			uri_base => $uri,
			dbname => $r->dir_config('dbname'),
			dbuser => $r->dir_config('dbuser'),
			dbpass => $r->dir_config('dbpass'),
			dbhost => $r->dir_config('dbhost'),
			debug => $r->dir_config('debug'),
			iface => $r->dir_config('iface') || 'eth0',
			lang => $r->dir_config('lang'),
			tmpldir => $r->dir_config('tmpldir'),
			params => \%params,
			remote_ip => $conn->remote_ip,
			session => $session);

    $ip->process;

    if (my $session = $ip->session) {
	$cookie = CGI::Cookie->new(-name=>'ipmanage',
				   -value=>$session);
	$r->headers_out->set('Set-Cookie' => $cookie->as_string);
    }
    $r->no_cache(1);
    $r->content_type($ip->content_type);
    print($ip->get_output);

    return OK;
}

1;
